"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.convertTypeAnnotationToFlowType = convertTypeAnnotationToFlowType;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function flowTypeAnnotationToString(type, nullable) {
  switch (type.type) {
    case "BooleanTypeAnnotation":
      return { type: "boolean", nullable: nullable };
    case "NumberTypeAnnotation":
      return { type: "number", nullable: nullable };
    case "StringTypeAnnotation":
      return { type: "string", nullable: nullable };
    default:
      return { type: "any", nullable: nullable };
  }
}

// convert an ObjectTypeAnnotation to a js object

function convertFlowObjectTypeAnnotation(objectType, flowTypes) {
  return objectType.properties.reduce(function (obj, property) {
    var key = property.key.name;

    return _extends({}, obj, _defineProperty({}, key, convertTypeAnnotationToFlowType(property.value, property.optional, flowTypes)));
  }, {});
}

function convertTypeAnnotationToFlowType(value, nullable, flowTypes) {
  if (value.type === "NullableTypeAnnotation") {
    return convertTypeAnnotationToFlowType(value.typeAnnotation, true, flowTypes);
  }

  if (value.type === "GenericTypeAnnotation" && flowTypes[value.id.name]) {
    return convertTypeAnnotationToFlowType(flowTypes[value.id.name], nullable, flowTypes);
  }

  if (value.type === "ObjectTypeAnnotation") {
    return {
      type: "object",
      nullable: nullable,
      properties: convertFlowObjectTypeAnnotation(value, flowTypes)
    };
  }

  if (value.type === "ArrayTypeAnnotation") {
    return {
      type: "array",
      nullable: nullable,
      child: convertTypeAnnotationToFlowType(value.elementType, nullable, flowTypes)
    };
  }

  return flowTypeAnnotationToString(value, nullable);
}