"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.calculateFragmentOptions = calculateFragmentOptions;


function objectExpressionProperties(node) {
  return node.properties.reduce(function (opts, property) {
    if (property.key.type === "Identifier") {
      opts[property.key.name] = property.value;
    }
    return opts;
  }, {});
}
/* eslint no-param-reassign:0 */


function findStringLiteral(name, items) {
  var node = items[name];
  if (node && node.type === "StringLiteral") {
    return node.value;
  }
  return null;
}

function calculateFragmentOptions(callExpression) {
  var argumentIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

  var options = {};

  var optionObj = callExpression.arguments[argumentIndex];
  if (optionObj && optionObj.type === "ObjectExpression") {
    var fragmentOptions = objectExpressionProperties(optionObj);

    ["name", "type", "templateTag"].forEach(function (key) {
      var value = findStringLiteral(key, fragmentOptions);
      if (value) {
        options[key] = value;
      }
    });

    var directivesNode = fragmentOptions.directives;
    if (directivesNode && directivesNode.type === "ObjectExpression") {
      var rawDirectiveOptions = objectExpressionProperties(directivesNode);
      options.directives = Object.keys(rawDirectiveOptions).reduce(function (opts, directiveName) {
        var directiveArgsNode = rawDirectiveOptions[directiveName];
        if (directiveArgsNode.type === "ObjectExpression") {
          // find any arguments that go with the directive
          opts[directiveName] = directiveArgsNode.properties.reduce(function (args, property) {
            if (property.key.type === "Identifier") {
              var argName = property.key.name;
              switch (property.value.type) {
                case "BooleanLiteral":
                case "NumericLiteral":
                case "StringLiteral":
                  args[argName] = property.value.value;
                  break;
                default:
                // nothing
              }
            }
            return args;
          }, {});
        }
        return opts;
      }, {});
    }
  }

  return options;
}