"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseReactComponentClass = parseReactComponentClass;
function extendsReactComponent(node) {
  var superClass = node.superClass;
  if (superClass && superClass.type === "MemberExpression") {
    return superClass.object.name === "React" && (superClass.property.name === "Component" || superClass.property.name === "PureComponent");
  }
  if (superClass && superClass.type === "Identifier") {
    return superClass.name === "Component" || superClass.name === "PureComponent";
  }
  return false;
}

function findClassProperty(node, propertyName) {
  var body = node.body.body;
  for (var i = 0; i < body.length; i += 1) {
    var bodyNode = body[i];
    if (bodyNode.type === "ClassProperty" && bodyNode.key.name === propertyName) {
      return bodyNode;
    }
  }
  return null;
}

function parseReactComponentClass(node) {
  if (!extendsReactComponent(node)) {
    return null;
  }

  var result = function result(n) {
    return {
      className: node.id.name,
      propType: n.id.name
    };
  };

  var typeParamAnnotation = node.superTypeParameters && node.superTypeParameters.params[1];
  if (typeParamAnnotation && typeParamAnnotation.type === "GenericTypeAnnotation") {
    return result(typeParamAnnotation);
  }

  var propsClassProperty = findClassProperty(node, "props");
  if (propsClassProperty) {
    var typeAnnotation = propsClassProperty.typeAnnotation && propsClassProperty.typeAnnotation.typeAnnotation;
    if (typeAnnotation && typeAnnotation.type === "GenericTypeAnnotation") {
      return result(typeAnnotation);
    }
  }

  return null;
}